!!ARBfp1.0
# computes 4D noise based on 2D texture containing
# 2D noise in red, z gradient in green, and w gradient in blue

ATTRIB frame = fragment.texcoord[0];
ATTRIB noiseInput = fragment.texcoord[1];

# base for blum blum shub should be product of two primes
# but I couldn't find any product that looked good and didn't
# result in huge texture sizes. 61 is a single prime that seemed 
# to produce reasonably good results
PARAM base = {61., 0.0163934}; # base, 1/base

# common scale & bias factors
PARAM s = {2,-1};

# integer and fractional components of input Z and W
TEMP intArg, fracArg;
FLR intArg.xy, noiseInput.zwzw;
FRC fracArg.xy,noiseInput.zwzw;

# compute fade values from fractional location in grid cell
# fade(t) = t*t*t*(t*(6*t-15)+10)
TEMP fade;
MAD fade.xy,fracArg,-2,3;		# old fade function t*t*(-2*t+3)
MUL fade.xy,fade,fracArg;
MUL fade.xy,fade,fracArg;

# hash will hold values for z,w; z+1,w; z,w+1 and z+1,w+1
TEMP hash, htmp;

# start with w & w+1 terms and run through hash function
MAD hash.xy,intArg.y,1,{0,1};	# {y,y+1}
MUL hash.xy,hash,hash;		#   squared
MUL htmp.xy,hash,base.y;	#   mod base
FLR htmp.xy,htmp;
MAD hash.xy,htmp,-base.x,hash;

# add z & z+1 terms and run through hash again
MAD hash,intArg.x,3,hash.xxyy;	# +3*{z,z+1,z,z+1}}
ADD hash,hash,{0,3,0,3};
MUL hash,hash,hash;		#   squared
MUL hash,hash,base.y;		#   cheap mod w/ 0..1 result
FRC hash,hash;

# compute texture coordinates for x/y slices to combine
# t<i><j> = z+j,w+i
TEMP t00,t01,t10,t11;
MOV t00.x,noiseInput; MOV t01.x,noiseInput; 
MOV t10.x,noiseInput; MOV t11.x,noiseInput;

ADD t00.y,noiseInput.y,hash.x; ADD t01.y,noiseInput.y,hash.y; 
ADD t10.y,noiseInput.y,hash.z; ADD t11.y,noiseInput.y,hash.w;

# look up slices
TEMP n00, n01, n10, n11;
TEX n00.xyz,t00,texture[0],2D; TEX n01.xyz,t01,texture[0],2D; 
TEX n10.xyz,t10,texture[0],2D; TEX n11.xyz,t11,texture[0],2D; 

# re-sort into xy, z & w components, each with one term from each slice
TEMP n, nz, nw;
MAD n.x, n00.x,s.x,s.y; MAD n.y, n01.x,s.x,s.y;
MAD n.z, n10.x,s.x,s.y; MAD n.w, n11.x,s.x,s.y;

MAD nz.x,n00.y,s.x,s.y; MAD nz.y,n01.y,s.x,s.y;
MAD nz.z,n10.y,s.x,s.y; MAD nz.w,n11.y,s.x,s.y;

MAD nw.x,n00.z,s.x,s.y; MAD nw.y,n01.z,s.x,s.y;
MAD nw.z,n10.z,s.x,s.y; MAD nw.w,n11.z,s.x,s.y;


# re-combine with additional gradient terms
TEMP grad, noise,n0,n1;
ADD grad,fracArg.xxyy,{0,-1,0,-1};
MAD n,nz,grad.xyxy,n;
MAD n,nw,grad.zzww,n;

# blend slices
LRP n.xy,fade.x,n.ywyw,n.xzxz;
LRP noise.x,fade.y,n.y,n.x;

# combine with diffuse lighting
MAD noise.x,noise.x,.5,.5;
MUL result.color.rgb,noise.x,fragment.color;
MOV result.color.a,1;
END