!!ARBvp1.0
# vertex shader for 3D turbulence-based wood
#
# keys: 
#   1/2 (object red) controls amount of turbulence (default none)


PARAM nscale = .05;		  # overall scale
PARAM oscale = {61,122,244,488};  # 61 (noise size) * {1,2,4,8} (octave scales)
PARAM wscale = {-7,5,4,2};	  # xyz=bias, w=scale to control wood rings
PARAM L = {-.577,.577,.577,0};	  # light position
PARAM Kd = {.9,.7,.5,0};	  # diffuse color
PARAM Ka = {.1,.14,.18,1};	  # ambient color

# inputs and outputs
ATTRIB Vm = vertex.position;	        # position in model space
ATTRIB Nm = vertex.normal;	        # normal in model space
OUTPUT Vp = result.position;	        # position in projection space
OUTPUT frame = result.texcoord[0];	# user input
OUTPUT noiseInput1 = result.texcoord[1]; # 1st octave coordinate (scale=1)
OUTPUT noiseInput2 = result.texcoord[2]; # 2nd octave coordinate (scale=2)
OUTPUT noiseInput4 = result.texcoord[3]; # 3rd octave coordinate (scale=4)
OUTPUT noiseInput8 = result.texcoord[4]; # 4th octave coordinate (scale=8)
OUTPUT noiseInputZ = result.texcoord[5]; # z for octaves 1-4
OUTPUT woodSpace   = result.texcoord[6]; # w/ scale & bias to control rings

# arbitrary user control
# include some scaling to put into a range where interesting things happen
MAD frame,vertex.color,{.05,1,1,1},{0,1,0,0};

# Transform the model-space vertex by the model to projection matrix
DP4 Vp.x, state.matrix.mvp.row[0], Vm;
DP4 Vp.y, state.matrix.mvp.row[1], Vm;
DP4 Vp.z, state.matrix.mvp.row[2], Vm;
DP4 Vp.w, state.matrix.mvp.row[3], Vm;

# Transform the normal to view space
TEMP N;
DP3 N.x,state.matrix.modelview.row[0],Nm;
DP3 N.y,state.matrix.modelview.row[1],Nm;
DP3 N.z,state.matrix.modelview.row[2],Nm;

# compute per-vertex diffuse shading
TEMP Nn,diff;
DP3 Nn.w,N,N;
RSQ Nn.w,Nn.w;
MUL Nn,N,Nn.w;
DP3 diff.x,Nn,{-.577,.577,.577,0};
MAX diff.x,diff.x,0;
MAD result.color,diff.x,{.9,.7,.5,0},{.1,.14,.18,1};

# pass in the noise inputs
TEMP ni;
MUL ni, vertex.position, nscale;
MUL noiseInputZ, ni.z, oscale;
MUL noiseInput1, ni.xyyy, 1;
MUL noiseInput2, ni.xyyy, 2;
MUL noiseInput4, ni.xyyy, 4;
MUL noiseInput8, ni.xyyy, 8;
MAD woodSpace,   ni,wscale.w,wscale;

END
